const config = require('config')
const helpers = require('./helpers')
const SendEmail = require('@pubsweet/component-send-email')
// const mailService = require('pubsweet-component-mail-template-service')

class Email {
  constructor({
    type = 'system',
    toUser = {
      id: '',
      email: '',
      name: '',
    },
    content = {
      subject: '',
      titleText: '',
      signatureName: '',
      comments: '',
      detailsLink: '',
      sourceUserName: '',
      unsubscribeLink: '',
      timestamp: '',
    },
  }) {
    this.type = type
    this.toUser = toUser
    this.content = content
  }

  set _toUser(newToUser) {
    this.toUser = newToUser
  }

  set _subject(newSubject) {
    this.subject = newSubject
  }

  set _content(newContent) {
    this.content = newContent
  }
  getBody({ emailType }) {
    const replacements = {
      signatureName: this.content.signatureName,
      toUserName: '',
      beforeAnchor: 'Please visit the',
      afterAnchor: 'to see the full review',
      detailsLink: this.content.detailsLink,
      unsubscribeLink: this.content.unsubscribeLink,
    }
    switch (emailType) {
      case 'unassign-reviewer':
        replacements.paragraph = `You are no longer needed to review ${
          this.titleText
        }. If you have comments on this manuscript you believe the Editor should
          see, please email them to ${config.get(
            'mailer.from',
          )} as soon as possible. Thank you for your
          time and I hope you will consider reviewing for Hindawi again.`
        break
      case 'review-submitted':
        replacements.paragraph = `We are pleased to inform you that Dr. ${
          this.sourceUserName
        } has submitted a review for ${this.titleText}.`
        break
      case 'reviewer-agreed':
        // subject = `${meta.collection.customId}: Manuscript Reviews`
        replacements.paragraph = `We are pleased to inform you that Dr. ${
          this.sourceUserName
        } has agreed to review ${this.titleText}. <br/>
        You should receive the report by Dr. ${
          this.sourceUserName
        } before ${helpers.getExpectedDate(this.content.timestamp, 14)}.`
        replacements.beforeAnchor =
          'If you have any queries, or would like to send a reminder if you no report has been submitted, then please visit the'
        break
      case 'reviewer-declined':
        // subject = `${meta.collection.customId}: Manuscript Reviews`
        replacements.paragraph = `We regret to inform you that Dr. ${
          this.sourceUserName
        } has declined to review ${this.titleText}.`
        replacements.afterAnchor =
          'to see if you need to invite any additional reviewers in order to reach a decision on the manuscript'
        break
      case 'reviewer-thank-you':
        // subject = `${meta.collection.customId}: Manuscript Review`
        replacements.paragraph = `Thank you for agreeing to review ${
          this.titleText
        }.`
        replacements.beforeAnchor =
          'You can view the full PDF file of the manuscript and post your review report using the following URL:'
        replacements.afterAnchor = ''
        break
      case 'agreed-reviewers-after-he-recommendation':
        replacements.hasLink = false
        replacements.paragraph = `I appreciate any time you may have spent reviewing ${
          this.titleText
        }. However, an editorial decision has been made and the review of this manuscript is now complete. I apologize for any inconvenience. <br/>
        If you have comments on this manuscript you believe the Editor should see, please email them to Hindawi as soon as possible. <br/>
        Thank you for your interest and I hope you will consider reviewing for Hindawi again.`
        break
      case 'pending-reviewers-after-he-recommendation':
        replacements.hasLink = false

        replacements.paragraph = `An editorial decision has been made regarding ${
          this.titleText
        }. So, you do not need to proceed with the review of this manuscript. <br/><br/>
        If you have comments on this manuscript you believe the Editor should see, please email them to Hindawi as soon as possible.`
        break
      case 'eic-recommendation':
        // subject = `${meta.collection.customId}: Manuscript Recommendation`
        replacements.beforeAnchor =
          'For more information about what is required, please visit the '
        break
      case 'author-request-to-revision':
        replacements.paragraph = `In order for ${
          this.titleText
        } to proceed to publication, there needs to be a revision. <br/><br/>
        ${this.content.comments}<br/><br/>`
        replacements.beforeAnchor =
          'For more information about what is required, please visit the '
        break
      case 'author-manuscript-rejected':
        replacements.paragraph = `I am sorry to inform you that ${
          this.titleText
        } has been rejected for publication. <br/><br/>
        ${this.content.comments}<br/><br/>`
        replacements.hasLink = false
        break
      case 'author-manuscript-published':
        replacements.paragraph = `I am delighted  to inform you that ${
          this.titleText
        } has passed through the review process and will be published in Hindawi.<br/><br/>
        ${this.content.comments}<br/><br/>
        Thanks again for choosing to publish with us.`
        replacements.hasLink = false
        break
      case 'he-manuscript-rejected':
        // subject = meta.emailSubject
        replacements.hasLink = false
        replacements.paragraph = `Thank you for your recommendation to reject ${
          this.titleText
        } based on the reviews you received.<br/><br/>
        I can confirm this article has now been rejected.`
        break
      case 'he-manuscript-published':
        // subject = meta.emailSubject
        replacements.hasLink = false
        replacements.paragraph = `Thank you for your recommendation to publish the manuscript titled ${
          this.titleText
        } based on the reviews you received.<br/><br/>
        I can confirm this article will now go through to publication.`
        break
      case 'he-manuscript-return-with-comments':
        // subject = meta.emailSubject
        replacements.hasLink = false
        replacements.paragraph = `Thank you for your recommendation for ${
          this.titleText
        } based on the reviews you received.<br/><br/>
        ${this.content.comments}<br/><br/>`
        break
      case 'submitting-reviewers-after-decision':
        replacements.hasLink = false
        replacements.paragraph = `Thank you for your review on ${
          this.titleText
        }. After taking into account the reviews and the recommendation of the Handling Editor, I can confirm this article ${
          this.content.comments
        }.<br/><br/>
        If you have any queries about this decision, then please email them to Hindawi as soon as possible.`
        break
      case 'new-version-submitted':
        replacements.paragraph = `A new version of ${
          this.titleText
        } has been submitted.`
        replacements.beforeAnchor =
          'Previous reviewers have been automatically invited to review the manuscript again. Please visit the'
        replacements.afterAnchor =
          'to see the latest version and any other actions you may need to take'
        break
      case 'submitting-reviewers-after-revision':
        replacements.paragraph = `A new version of ${
          this.titleText
        } has been submitted.`
        replacements.beforeAnchor = `As you have reviewed the previous version of this manuscript, I would be grateful if you can review this revised version and submit a review report by ${helpers.getExpectedDate(
          this.content.timestamp,
          14,
        )}. You can download the PDF of the revised version and submit your new review from the following URL:`
        break
      default:
        throw new Error(`undefined email type: ${emailType}`)
    }

    return {
      html: helpers.getNotificationBody('notification', replacements),
      text: `${replacements.intro} ${replacements.paragraph} ${
        replacements.beforeAnchor
      } ${replacements.detailsUrl} ${replacements.afterAnchor} ${
        replacements.signatureName
      }`,
    }
  }

  sendEmail({ text, html }) {
    const mailData = {
      from: config.get('mailer.from'),
      to: this.toUser.email,
      subject: this.content.subject,
      text,
      html,
    }

    SendEmail.send(mailData)
  }
}

module.exports = Email
